#!/usr/bin/perl

package Cognition::Export::Entity;
use CGI::Util;
use Cognition::Export::Alarm;
use Cognition::Export::BlogEntry;
use Cognition::Export::Calendar;
use Cognition::Export::Contact;
use Cognition::Export::Event;
use Cognition::Export::FBInfo;
use Cognition::Export::Feed;
use Cognition::Export::Location;
use Cognition::Export::Recording;
use Cognition::Export::Todo;
use Cognition::Export;
use Cognition::MagicString;
use Cognition::RDFModel::Property;
use Cognition::RDFModel::Subject;
use Cognition::RDFModel::Value;
use Digest::SHA1 qw(sha1_hex);
use HTML::Entities qw(encode_entities_numeric);
use JSON 2.0;
use strict;

our $Encounters = {};

sub is
{
	my $subject = shift;
	return unless ($subject);
	return 1;
}

sub outputs
{
	return [
		{
			code  => 'RDF',
			name  => 'RDF/XML',
			type  => 'application/rdf+xml',
			multi => 1
		},
		{
			code  => 'Turtle',
			name  => 'RDF/Turtle',
			type  => 'application/rdf+turtle',
			multi => 1
		},
		{
			code  => 'JSON',
			name  => 'RDF/JSON',
			type  => 'application/x-rdf+json',
			multi => 1
		},
		{
			code  => 'Detect',
			name  => 'Detection Page',
			type  => 'text/html',
			multi => 1
		},
		{
			code  => 'Title',
			name  => 'Entity Title',
			type  => 'text/plain'
		}
	];
}


sub outTitle
{
	my $triples = $_[0];
	my $S       = $_[1];	
	my $subject = findSubject($triples, $S);

	my $t = firstOfLiteral($subject,
		['http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title',
		'urn:ietf:rfc:2426#fn',
		'http://www.w3.org/2006/vcard/ns#fn',
		'http://www.w3.org/2006/03/hcard#fn',
		'http://www.w3.org/2001/vcard-rdf/3.0#FN',
		'http://xmlns.com/foaf/0.1/fullName',
		'http://xmlns.com/foaf/0.1/name',
		'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
		'urn:ietf:rfc:2426#street-address',
		'urn:ietf:rfc:4287#title',
		'http://purl.org/rss/1.0/title',	
		'urn:ietf:rfc:2445#summary',
		'http://www.w3.org/2002/12/cal/ical#summary',
		'urn:ietf:rfc:2445#x-wr-calname'],
		['http://www.w3.org/1999/02/22-rdf-syntax-ns#value',
		'urn:ietf:rfc:2445#value',
		'urn:ietf:rfc:2426#value',]);
	
	return $t if (length $t);
	return $1 if ($S =~ /^bnode\:\/\/\/(.*)$/);
	return $S;
}


sub multiDetect
{
	my $triples  = shift;
	my @subjects = keys %$triples;
	my $page;
	while (!$page && @subjects)
	{
		my $subj = shift @subjects;
		$page = $triples->{$subj}->{page};
	}
	
	my $rv;
	$rv .= "<div class=\"hRDF\" typeof=\"hrdf:Block\" id=\"hRDF-page\">\n";
	$rv .= "<h2>Page: <span class=\"fn\" property=\"dc:title\">"
		.encode_entities_numeric(outTitle($triples, $page->abs_url))
		."</span></h2>\n";
	$rv .= "<ul>\n";
	my $uri = $page->abs_url;
	$uri =~ s/^http\:\/\///;
	$uri = encode_entities_numeric(CGI::Util::escape(CGI::Util::escape($uri)));
	$uri =~ s/\%252F/\//ig;
	foreach my $k (qw(Entity+rdf-xml+RDF/XML Entity+rdf-json+RDF/JSON
		Entity+turtle+Turtle Contact+vcard+vCard Contact+jcard+jCard
		Location+kml+KML Calendar+icalendar+iCalendar Recording+m3u+M3U))
	{
		my ($kind, $path, $format) = split /\+/, $k;
		$kind = lc($kind).'s';
		$kind =~ s/ys$/ies/;
		$kind = "All $kind";
		$kind = "Page" if ($kind eq 'All entities');
		$rv .= "\t<li><a rel=\"xhv:alternate\" href=\"http://srv.buzzword.org.uk/$path/$uri\">$kind as $format</a></li>\n";
	}
	$rv .= "</ul>\n";
	$rv .= "</div>\n";
	$rv .= "<h2>Items Found on Page</h2>\n";
	foreach my $s (sort SubjectSort (keys %$triples))
	{
		$rv .= outDetect($triples, $s, 'h3', 1);
	}
	return $rv;
}


sub outDetect
{
	my $triples = shift;
	my $S       = shift;	
	my $htag    = shift || 'h2';
	my $dolink  = shift || 0;
	my $subject = findSubject($triples, $S);
	my $rv;
	my $links   = _outDetect_links($triples, $S);
	my $turtle  = outTurtle($triples, $S, 'html', $dolink);
	my $id      = 'hRDF-'.sha1_hex($S);
	return unless (length $turtle);

	$rv .= "<div class=\"hRDF\" typeof=\"hrdf:Block\" id=\"$id\">\n";
	$rv .= "<$htag class=\"fn\" property=\"dc:title\">".encode_entities_numeric(outTitle($triples, $S))."</$htag>\n";
	$rv .= "<div class=\"url\">&lt;<span class=\"value\">$S</span>&gt;</div>\n" unless $S =~ /^bnode/;
	$rv .= "<pre class=\"data\" property=\"hrdf:data\" datatype=\"hrdf:turtle\">";
	$rv .= $turtle;
	$rv .= "</pre>\n";
	$rv .= $links if (length $links);
	$rv .= "</div>\n";
	
	return $rv;
}

#	my $rv;
#	if ($S eq '-')
#		{ $rv = "\t<dt>(All on page)</dt>\n"; }
#	else
#		{ $rv = "\t<dt>".encode_entities_numeric(outTitle($triples, $S))."</dt>\n"; }

sub _outDetect_links
{
	my $triples = $_[0];
	my $S       = $_[1];
	
	my $subject = findSubject($triples, $S);
	my $uri;

	if ($subject->{page})
	{
		$uri = $subject->{page}->abs_url;
		if ($uri.'#' eq substr($S, 0, length($uri)+1))
		{
			$uri = $S;
		}
		else
		{
			$uri .= "#subject($S)";
		}
	}
	else
	{
		return undef;
	}
	$uri =~ s/^http\:\/\///;
	$uri = encode_entities_numeric(CGI::Util::escape(CGI::Util::escape($uri)));
	$uri =~ s/\%252F/\//ig;
	
	my $rv = "<ul>\n";
	foreach my $k (qw(Entity+rdf-xml+RDF/XML Entity+rdf-json+RDF/JSON 
		Entity+turtle+Turtle Contact+vcard+vCard Contact+jcard+jCard Location+kml+KML
		Calendar+icalendar+iCalendar Feed+atom+Atom Feed+icalendar+iCalendar
		Event+icalendar+iCalendar BlogEntry+icalendar+iCalendar
		Alarm+icalendar+iCalendar FBInfo+icalendar+iCalendar
		Todo+icalendar+iCalendar Recording+m3u+M3U))
	{
		my ($kind, $path, $format) = split /\+/, $k;
		
		if (
			($S eq '-')
		||	($kind eq 'Contact' && Cognition::Export::Contact::is($subject))
		||	($kind eq 'Entity' && Cognition::Export::Entity::is($subject))
		||	($kind eq 'Location' && Cognition::Export::Location::is($subject))
		||	($kind eq 'Calendar' && Cognition::Export::Calendar::is($subject))
		||	($kind eq 'Event' && Cognition::Export::Event::is($subject))
		||	($kind eq 'BlogEntry' && Cognition::Export::BlogEntry::is($subject))
		||	($kind eq 'Recording' && Cognition::Export::Recording::is($subject))
		||	($kind eq 'Alarm' && Cognition::Export::Alarm::is($subject))
		||	($kind eq 'FBInfo' && Cognition::Export::FBInfo::is($subject))
		||	($kind eq 'Todo' && Cognition::Export::Todo::is($subject))
		||	($kind eq 'Feed' && Cognition::Export::Feed::is($subject))
		)
		{
			$rv .= "\t<li><a rel=\"xhv:alternate\" href=\"http://srv.buzzword.org.uk/$path/$uri\">$kind as $format</a></li>\n";
		}
	}
	$rv .= "</ul>\n";
	
	return $rv;
}


sub multiJSON
{
	my $triples = shift;
	my %rv;
	foreach my $s (keys %$triples)
	{
		my $snice = $s;
		$snice = "_:$1" if ($s =~ m|^bnode:/*(.*)|);
		my $r   = _outJSON($triples, $s);
		$rv{$snice} = $r if ($r);
	}
 	return to_json(\%rv, {utf8 => 1, pretty => 1, canonical => 1, convert_blessed => 1})."\n";
}

sub outJSON
{
	my $triples = $_[0];
	my $S       = $_[1];
	my $json = _outJSON($triples, $S);
	my %rv;
	my $snice   = $_[1];
	$snice      = "_:$1" if ($_[1] =~ m|^bnode:/*(.*)|);
	$rv{$snice} = $json;

 	return to_json(\%rv, {utf8 => 1, pretty => 1, canonical => 1, convert_blessed => 1})."\n";
}

sub multiTurtle
{
	my $rv;
	my $triples = shift;
	my $pfxs = {};
	
	foreach my $s (sort SubjectSort (keys %$triples))
	{
		my $R = outTurtle($triples, $s);
		if (length $R)
		{
			$R =~ m/^(.*?)\n\n(.*)$/s;
			$rv .= "$2\n";
			
			my @p = split /\n/, $1;
			foreach my $p (@p)
				{ $pfxs->{$p} = 1; }
		}
	}

	return join("\n", (sort keys %$pfxs))."\n\n".$rv;
}

sub _optEnt
{
	my $str = shift;
	my $fmt = shift;
	
	return encode_entities_numeric($str)
		if ($fmt eq 'html');
	return $str;
}

sub outTurtle
{
	my $triples = shift;
	my $S       = shift;
	my $format  = shift || 'text';
	my $dolink  = shift || 0;
	my $subject = findSubject($triples, $S);
	my $this    = $subject->{page};
	my $json    = _outJSON($triples, $S);

	return undef unless (keys %$json);

	my $rv;
	my $pfxs = {};
	if ($S =~ m|^bnode:/*(.*)|)
		{ $rv = _optEnt("_:$1 ",$format); }
	else
		{ $rv = _optEnt("<$S> ",$format); }
		
	foreach my $predicate (sort PredicateSort (keys %$json))
	{
		next if ($predicate eq 'http://www.w3.org/2006/link#uri');
		my $prednice = $this->_fq2pfx($predicate);
		$prednice = "<$predicate>" unless ($prednice);
		next unless ($prednice =~ /[\:\/]/);
		$prednice = 'a' if ($prednice eq 'rdf:type');
		
		$pfxs->{ _optEnt(sprintf("\@prefix %-9s <%s> .", $this->{NamespacePrefixes}->{ $1 }.':', $1), $format) } = 1
			if ($predicate =~ /^(.*[\#\/])([^\#\/]*)$/);
		
		if ($format eq 'html')
		{
			$rv .= '<b>'.encode_entities_numeric($prednice).'</b> ';
		}
		else
		{
			$rv .= "$prednice ";
		}
		
		foreach my $value (@{ $json->{$predicate} })
		{
			my $valnice = '';
			if ($value->{type} eq 'uri')
			{
				$value->{value} =~ s/^_:/bnode:\/\/\//;
			
				my $vURI;
				my $vSubject;
				if ($dolink && ($format eq 'html'))
				{
					$vSubject = findSubject($triples, $value->{value});
					$vURI     = '#hRDF-'.sha1_hex($vSubject->uri) if ($vSubject);
				}

				$valnice = $this->_fq2pfx($value->{value});
				
				# BNodes
				if ($value->{value} =~ /^bnode\:\/*(.*)$/)
				{
					if (length $vURI)
						{ $valnice = "<i><a href=\"$vURI\">"._optEnt("_:$1", $format)."</a></i>"; }
					else
						{ $valnice = ($format eq 'html') ? ('<i>'._optEnt("_:$1", $format).'</i>') : "_:$1"; }
				}
				# CURIEs
				elsif ($valnice =~ /.+\:.+/)
				{
					$valnice = ($format eq 'html') ? ('<i>'._optEnt($valnice, $format).'</i>') : $valnice;

					$pfxs->{ _optEnt(sprintf("\@prefix %-9s <%s> .", $this->{NamespacePrefixes}->{ $1 }.':', $1), $format) } = 1
						if ($value->{value} =~ /^(.*[\#\/])([^\#\/]*)$/ && defined $this->{NamespacePrefixes}->{$1});
				}
				# URIs
				else
				{
					$valnice = $value->{value};
					if (length $vURI)
						{ $valnice = "&lt;<a href=\"$vURI\">"._optEnt($valnice, $format)."</a>&gt;"; }
					else
						{ $valnice = _optEnt("<$valnice>", $format); }
				}
			}
			else
			{
				$valnice = $value->{value};
				$valnice =~ s/\\/\\\\/g;
				$valnice =~ s/\"/\\\"/g;
				$valnice = "\"$valnice\"";
				$valnice .= '@'.$value->{lang}			
					if (length $value->{lang});
				$valnice = _optEnt($valnice, $format);

				if (length $value->{datatype})
				{
					my $dtnice = $this->_fq2pfx($value->{datatype});
					if (length $dtnice)
					{
						$pfxs->{ _optEnt(sprintf("\@prefix %-9s <%s> .", $this->{NamespacePrefixes}->{ $1 }.':', $1), $format) } = 1
							if ($value->{datatype} =~ /^(.*[\#\/])([^\#\/]*)$/);
					}
					else
					{
						$dtnice = sprintf('<%s>', $value->{datatype});
					}
					
					$valnice .= '^^'.$dtnice;
				}
			}
			
			$rv .= $valnice . " ,\n\t\t";
			
		}
		
		$rv =~ s/\,\n\t\t$/;\n\t/;
	}

	$rv =~ s/\;\n\t$/.\n/;

	return join("\n", (sort keys %$pfxs))."\n\n".$rv;
}

sub _outJSON
{
	my $triples = $_[0];
	my $S       = $_[1];
	my $subject = findSubject($triples, $S);
	my $rv      = {};
	
	my $return_null = 1;

	foreach my $prop (values %{$subject->{properties}})
	{
		$return_null = 0;

		my @values;
		foreach my $val (@{$prop->valObjects})
		{
			next if ($prop->uri eq 'http://www.w3.org/2006/link#uri'
				&& $val->toString =~ /^bnode:/);

			my $V = {};
			$V->{type}  = $val->type ? 'uri' : 'literal';
			$V->{value} = $val->toString;
			if (isms($val->toString))
			{
				$V->{lang}  = $val->toString->{lang};
				$V->{value} = $val->toString->{string};
			}
			elsif ((ref $val->toString) =~ /^DateTime/i)
			{
				$V->{value} = "".$val->toString;
			}
			if ( $val->datatype )     { $V->{datatype} = $val->datatype; }
			
			if ($val->type && $val->toString =~ m|^bnode:/*(.*)|)
				{ $V->{value} = "_:$1"; }
			
			push @values, $V;
		}
		
		$rv->{ $prop->uri } = \@values if (@values);
	}

	return undef if ($return_null);
	
	return $rv;
}


sub _outRDF
{
	my $triples = $_[0];
	my $S       = $_[1];
	my $subject = findSubject($triples, $S);
	my $this    = $subject->{page};
	my $rv = '';

	return unless ($subject && (ref $subject eq 'Cognition::RDFModel::Subject'));
	
	$Cognition::Export::Entity::Encounters{$S} = 1;

	my $tag = 'rdf:Description';
	my $rdfHeadType = firstOf($subject, 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type');
	if ($rdfHeadType)
		{ $tag = $this->_fq2pfx($rdfHeadType); }

	$rv .= sprintf("\t<$tag%s>\n", rdf_about($triples, $subject->uri, 1));
	foreach my $property ($subject->propertiesObjects)
	{
		next if ($property->nVals == 0);
		next unless (length $property->namespace);
		my $pfxprop = $this->_fq2pfx($property->uri);
		
		foreach my $val (@{ $property->valObjects })
		{
			next if ($property->uri eq 'http://www.w3.org/2006/link#uri'
				&& $val->toString =~ /^bnode:/);

			next if ($property->uri eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type'
				&& $val->toString eq $rdfHeadType);
				
			my $langattr = '';
			if (isms($val->toString) && $val->toString->{lang})
				{ $langattr = sprintf(" xml:lang=\"%s\"", encode_entities_numeric($val->toString->{lang})); }

			my $datatypeattr = '';
			if ( $val->datatype )
				{ $datatypeattr = sprintf(" rdf:datatype=\"%s\"", encode_entities_numeric($val->datatype)); }
			
			my $xmlnsattr = '';
			my $valstr;
			if ($val->datatype eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral' && !$val->type)
			{
				$valstr = $val->toString;
				$xmlnsattr = sprintf(" rdf:parseType=\"Literal\" xmlns=\"%s\"", encode_entities_numeric($val->{xmlns})); 
				$xmlnsattr = ' rdf:parseType="Literal"'
					unless (length $val->{xmlns});
			}
			else
			{
				$valstr = encode_entities_numeric($val->toString); 
			}

			if ($val->type == 1)
			{
				if (defined $triples->{$valstr}
				&& $valstr =~ m#^(bnode:|tag:|http...openurl.invalid)#
				&& !$Cognition::Export::Entity::Encounters{$valstr})
				{
					my $nested = _outRDF($triples, $valstr);
					$nested =~ s/^/\t\t/gm;
					$rv .= sprintf("\t\t<%s>\n%s\t\t</%s>\n", $pfxprop, $nested, $pfxprop);
				}
				else
				{
					$rv .= sprintf("\t\t<%s%s />\n", $pfxprop, rdf_resource($triples, $valstr));
				}
			}
			elsif (length $val->vocab)
			{
				my $pfxvocab = $this->_fq2pfx($val->vocab);
			
				$rv .= sprintf("\t\t<%s>\n\t\t\t<%s>\n\t\t\t\t<rdf:value%s%s%s>%s</rdf:value>\n\t\t\t</%s>\n\t\t</%s>\n", 
									$pfxprop, $pfxvocab, $langattr, $datatypeattr, $xmlnsattr, $valstr, $pfxvocab, $pfxprop);
			}
			else
			{
				$rv .= sprintf("\t\t<%s%s%s%s>%s</%s>\n", 
									$pfxprop, $langattr, $datatypeattr, $xmlnsattr, $valstr, $pfxprop);
			}
		}
	}
	$rv .= "\t</$tag>\n";
	
	foreach my $alias (@{$subject->
		property('http://www.w3.org/2006/link#uri')
		->valObjects})
	{
		next unless ($alias->type);
		my $A = $alias->toString;
		$rv .= sprintf("\n\t<rdf:Description%s>"
			           ."\n\t\t<%s%s />"
		              ."\n\t</rdf:Description>"
			           ."\n",
			           rdf_about($triples, $A, 1),
			           $this->_fq2pfx('http://www.w3.org/2002/07/owl#sameAs'),
			           rdf_resource($triples, $subject->uri, 1))
			unless ($A eq $S || $A =~ /\#xpointer\(/ || $A =~ /^bnode:/);
	}
	
	return $rv;
}


sub outRDF
{
	my $triples = $_[0];
	my $S       = $_[1];
	my $subject = $triples->{$S};
	my $this    = $subject->{page};

	$Cognition::Export::Entity::Encounters = {};

	my $rv = "<?xml version=\"1.0\"?>\n"
		. "<rdf:RDF";

	my (%used_namespaces);
	foreach my $p ( $subject->propertiesObjects )
	{
		$used_namespaces{$p->namespace} = 1;
	}
	
	foreach my $ns (keys %{ $this->{NS} })
	{
		my $nsurl = $this->{NS}->{$ns}->{nsurl};
		next unless (length $nsurl);
		$rv .= "\n\txmlns:".lc($ns)."=\"".encode_entities_numeric($nsurl)."\""
			if ($used_namespaces{$nsurl});
	}
	$rv .= "NS_PLACEHOLDER_99754106633f94d350db34d548d6091a>\n\n";
	$rv .= _outRDF($triples, $S);
	$rv .= "</rdf:RDF>\n";
	return add_namespaces($rv, $triples, $S);
}

sub multiRDF
{
	my $triples = $_[0];
	my @keys = keys %$triples;
	my $key = shift @keys;
	my $this = $triples->{$key}->{page};
	
	$Cognition::Export::Entity::Encounters = {};

	my $rv = "<?xml version=\"1.0\"?>\n"
		. "<rdf:RDF";

	foreach my $ns (keys %{ $this->{NS} })
	{
		my $nsurl = $this->{NS}->{$ns}->{nsurl};
		next unless (length $nsurl);
		$rv .= "\n\txmlns:".lc($ns)."=\"".encode_entities_numeric($nsurl)."\"";
	}
	$rv .= "NS_PLACEHOLDER_99754106633f94d350db34d548d6091a>\n\n";
	
	my @subjects = keys %$triples;
	my @sortedSubjects = sort SubjectSort @subjects;
	foreach my $S (@sortedSubjects)
	{
		unless ($Cognition::Export::Entity::Encounters{$S})
		{
			my $x = _outRDF($triples, $S);
			$rv .= "$x\n" if (length $x);
		}
	}
	
	$rv .= "</rdf:RDF>\n";
	return add_namespaces($rv, $triples);
}

sub add_namespaces 
{
	my $string  = shift;
	my $triples = shift;
	my $subject = shift;
	my $rv = '';
	my %already;
	my $page;
	
	if ($subject)
	{
		my $S = findSubject($triples, $subject);
		$page = $S->{page};
		
		foreach my $P ( $S->propertiesObjects )
		{
			my $ns = $P->namespace;
			next unless (length $ns);
			$rv .= sprintf("\n xmlns:%s=\"%s\"", $page->{'NamespacePrefixes'}->{$ns}, $ns)
				unless ($already{$page->{'NamespacePrefixes'}->{$ns}});
			$already{$page->{'NamespacePrefixes'}->{$ns}} = $ns;

			# TODO
			# Also need to check within values for vocabs as these will be
			# namespaced. Plus add OWL.
		}
	}
	else
	{
		foreach my $subject (keys %$triples)
		{
			my $S = $triples->{$subject};
			$page = $S->{page};

			next unless ($S && (ref $S eq 'Cognition::RDFModel::Subject'));
			
			foreach my $P ($S->propertiesObjects)
			{
				my $ns = $P->namespace;
				next unless (length $ns);
				$rv .= sprintf("\n xmlns:%s=\"%s\"", $page->{'NamespacePrefixes'}->{$ns}, $ns)
					unless ($already{$page->{'NamespacePrefixes'}->{$ns}});
				$already{$page->{'NamespacePrefixes'}->{$ns}} = $ns;

				if ($P->uri eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
				{
					foreach my $v (@{$P->valObjects})
					{					
						if ($v->type && $v->toString =~ /^(.*[\/\#])([^\/\#]+)/)
						{
							$rv .= sprintf("\n xmlns:%s=\"%s\"", $page->{'NamespacePrefixes'}->{$1}, $1)
								unless ($already{$page->{'NamespacePrefixes'}->{$1}});
							$already{$page->{'NamespacePrefixes'}->{$1}} = $1;
						}
					}
				}
			}
		}
	}
	
	$rv .= "\n xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\""
		unless ($rv =~ /1999.02.22\-rdf\-syntax\-ns/);

	$string =~ s/NS_PLACEHOLDER_99754106633f94d350db34d548d6091a/$rv/;
	
	return $string;
}

sub rdf_resource
{
	my $triples = shift;
	my $uri     = shift;
	my $sure    = shift;
	
	unless (1 || $sure)
	{
		my $subject = findSubject($triples, $uri);
		return '' unless ($subject && (ref $subject eq 'Cognition::RDFModel::Subject'));
		$uri = $subject->uri if ($subject);
	}
	
	if ($uri =~ m|^bnode:///(.*)$|)
		{ return sprintf(' rdf:nodeID="%s"', encode_entities_numeric($1)); }
		
	return sprintf(' rdf:resource="%s"', encode_entities_numeric($uri));
}

sub rdf_about
{
	my $triples = shift;
	my $uri     = shift;
	my $sure    = shift;
	
	unless ($sure)
	{
		my $subject = findSubject($triples, $uri);
		$uri = $subject->uri if ($subject);
	}
	
	if ($uri =~ m|^bnode:///(.*)$|)
		{ return sprintf(' rdf:nodeID="%s"', encode_entities_numeric($1)); }
		
	return sprintf(' rdf:about="%s"', encode_entities_numeric($uri));
}

sub SubjectSort
{
	return  1 if ($a =~ /^bnode/ && $b !~ /^bnode/);
	return -1 if ($b =~ /^bnode/ && $a !~ /^bnode/);
	
	return  1 if ($a =~ /\#/ && $b !~ /\#/);
	return -1 if ($b =~ /\#/ && $a !~ /\#/);
	
	$a cmp $b;
}

sub PredicateSort
{
	my $rdftype = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type';

	return -1 if ($a eq $rdftype && $b ne $rdftype);
	return  1 if ($a ne $rdftype && $b eq $rdftype);
	
	$a cmp $b;
}

1;
