#!/usr/bin/perl

package Cognition::Export::Todo;
use Cognition::Export;
use Cognition::Export::Event;
use MIME::Base64;

sub is
{
	my $subject = shift;
	return unless ($subject);
	return unless (ref $subject eq 'Cognition::RDFModel::Subject');	
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://www.w3.org/2002/12/cal/ical#Vtodo')
			||  ($type eq 'urn:ietf:rfc:2445#Vtodo'));
	}
	return;
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		}
	];
}

sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $rv = Cognition::Export::Event::_outICS(@_);
	
	$rv =~ s/^BEGIN\:VEVENT/BEGIN:VTODO/m;
	$rv =~ s/^END\:VEVENT/END:VTODO/m;
	
	return $rv;
}

1;
