#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::Metadata;
######################################################################

use Cognition::MagicString;
use XML::LibXML qw(:all);

sub is_enabled
{
	return 1;
}

sub consume
{
	my $this = shift;
	my $myurl = $this->abs_url;

	# Parse <title>.
	my $S = Cognition::RDFModel::Subject->new($myurl);
	foreach my $t ($this->{DOM}->getElementsByTagName('title'))
	{
		my $val = Cognition::RDFModel::Value->new(Cognition::HTMLParser::Utils::STRINGIFY($t));
		$val->setDom($t);
		$S->addVal('http://purl.org/dc/terms/title', $val);
	}
	my $val = Cognition::RDFModel::Value->new('http://purl.org/dc/dcmitype/Text', 1);
	$val->setVocab('http://purl.org/dc/terms/DCMIType');
	$S->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type', $val);
	$this->mergeSubject($S);
	
	meta_tags($this);
	link_tags($this);
}

sub link_tags
{
	my $this = shift;
	my $myurl = $this->abs_url('');
	
	# Get list of links.
	my @links   = $this->{DOM}->getElementsByTagName('link');
	my @anchors = $this->{DOM}->getElementsByTagName('a');
	push @links, @anchors;
	
	# For each link
	foreach my $l (@links)
	{
		# Subject: Linked page
		my $href = $this->abs_url($l->getAttribute('href'));
		next unless (length $href);
		my $SubjectLinkedPage;
		
		# Handle @title.
		if (length $l->getAttribute('title'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('title'));
			$val->setDom($l);
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/title', $val);
		}

		# Handle @media.
		if (length $l->getAttribute('media'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('media'));
			$val->setDom($l);
			$val->setVocab('http://www.w3.org/TR/CSS21/media.html#media-types');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/format', $val);
		}

		# Handle @title.
		if (length $l->getAttribute('type'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new($l->getAttribute('type'));
			$val->setDom($l);
			$val->setVocab('http://purl.org/dc/terms/IMT');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/format', $val);
		}

		# Handle @lang/@hreflang.
		if (length $l->getAttribute('lang')
		||  length $l->getAttribute('xml:lang')
		||  length $l->getAttribute('hreflang'))
		{
			$SubjectLinkedPage = Cognition::RDFModel::Subject->new($href)
				unless ($SubjectLinkedPage);

			my $val = Cognition::RDFModel::Value->new(
				$l->getAttribute('hreflang') ||
				$l->getAttribute('xml:lang') ||
				$l->getAttribute('lang')
			);
			$val->setDom($l);
			$val->setVocab('http://purl.org/dc/terms/RFC4646');
			$SubjectLinkedPage->addVal('http://purl.org/dc/terms/language', $val);
		}

		$this->mergeSubject($SubjectLinkedPage);
	}
}

sub meta_tags
{
	my $this = shift;
	my $myurl = $this->abs_url('');
	
	# Parse <meta>
	foreach my $m ($this->{DOM}->getElementsByTagName('meta'))
	{
		# Value
		my $content = length($m->getAttribute('content')) ?
		                   $m->getAttribute('content') :
		                   Cognition::HTMLParser::Utils::STRINGIFY($m);

		# Subject: This page
		my $SubjectThisPage = Cognition::RDFModel::Subject->new($myurl);
		
		# @http-equiv
		my $name = $m->getAttribute('http-equiv');
		if (length $name)
		{
			my $nsname = $this->_pfx2fq('HTTP11.'.$name, undef, $m);

			my $val = Cognition::RDFModel::Value->new($content);
			$val->setDom($m);
			$SubjectThisPage->addVal($nsname, $val);
		}
		
		# @name/@property
		else
		{
			$name = $m->getAttribute('property') . ' ' . $m->getAttribute('name');
			next unless (length $name);

			$name =~ s/(^\s+|\s+$)//g;
			$name =~ s/\s+/ /g;
			my @name = split / /, $name;
			
			foreach my $n (@name)
			{
				my $nsname = $this->_pfx2fq($n, undef, $m);
				
				my $val = Cognition::RDFModel::Value->new($content);
				$val->setDom($m);
				if ($m->getAttribute('scheme'))
				{
					my $nsscheme = $this->_pfx2fq($m->getAttribute('scheme'), undef, $m);
					$val->setVocab($nsscheme);
				}
				$SubjectThisPage->addVal($nsname, $val);
			}
		}
		
		$this->mergeSubject($SubjectThisPage);
	}
}

1;
