#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::RoleAttr;
######################################################################

use Cognition::MagicString;
use Cognition::HTMLParser::Utils;
use XML::LibXML qw(:all);

sub is_enabled
{
	return 1;
}

sub consume
{
	my $this = shift;
	my $myurl = $this->abs_url;
	
	# Parse @role
	foreach my $e ($this->{DOM}->getElementsByTagName('*'))
	{
		next unless (length $e->getAttribute('role'));
	
		# Subject: This page
		my $SubjectThisPage = Cognition::RDFModel::Subject->new($myurl);

		# Object: This element
		my $objuri;
		if (length $e->getAttribute('id'))
		{
			$objuri = $myurl . '#' . $e->getAttribute('id');
		}
		else
		{
			$objuri = Cognition::HTMLParser::Utils::BNodeURI($e);
		}

		# Predicate: role
		my $role = $e->getAttribute('role');
		$role =~ s/(^\s+|\s+$)//g;
		$role =~ s/\s+/ /g;
		my @role = split / /, $role;
		
		foreach my $r (@role)
		{
			# Predefined roles from the XHTML vocab namespace
			if ($r =~ /^\:?( banner | contentinfo | definition | main | navigation
				| note | search | secondary | seealso )$/xi)
			{
				$r = "xhv:$1"
			}
			my $nsrole = $this->_pfx2fq($r, undef, $e);
			
			my $val = Cognition::RDFModel::Value->new($objuri, 1);
			$val->setDom($e);
			$SubjectThisPage->addVal($nsrole, $val);
		}		
		
		$this->mergeSubject($SubjectThisPage);
	}
}

1;
